import sys
import logging
from threading import Thread

from .fetcher import *
from .custom_css import *

from .config import *
from .response import Message
from .messenger import Messenger

if sys.platform.startswith('win32'):
    from .channel.win.server import Server
else:
    from .channel.unix.server import Server

class Daemon:
    """
    Implements the daemon functionality that communicates with the extension.

    :param python_version str: the current major python version
    """
    def __init__(self, python_version):
        self.python_version = python_version
        self.set_chrome_path()
        self.messenger = Messenger(self.python_version)
        self.socket_server = Server()
        self.is_running = False

    def set_chrome_path(self):
        """Tries to set the path to the chrome directory."""
        self.chrome_path = get_firefox_chrome_path()

    def check_chrome_path(self, action):
        """
        Checks if the path to the 'chrome' directory was found and sends a message if it was not.

        :return: if chrome_path is set
        :rType: bool
        """
        if not self.chrome_path:
            self.messenger.send_message(Message(
                action,
                'Could not find path to chrome folder',
                success=False
            ))
            return False
        else:
            return True

    def check_target(self, message):
        """
        Checks if the message received specifies a target, or the message is invalid.

        :param message object: the decoded message
        :return: if message has key 'target' with a valid value
        :rType: bool
        """
        if 'target' in message and len(message['target']) > 0:
            return message['target']

        logging.error('%s: target was not specified' % message['action'])
        self.send_invalid_action()
        return False

    def send_version(self):
        """Sends the current daemon version to the extension."""
        self.messenger.send_message(Message(ACTIONS['VERSION'], data=DAEMON_VERSION))

    def send_colorscheme(self):
        """Sends the current colorscheme to the extension."""
        (success, colorscheme) = get_colorscheme()
        self.messenger.send_message(Message(ACTIONS['COLORS'], data=colorscheme, success=success))

    def send_invalid_action(self):
        """Sends an action to the extension indicating that the action sent was invalid"""
        self.messenger.send_message(Message(ACTIONS['INVALID_ACTION'], success=False))

    def send_output(self, message):
        """
        Sends an output message to the extension that will be displayed in the 'Debugging output' area.

        :param message str: the message to send to the extension
        """
        self.messenger.send_message(Message(ACTIONS['OUTPUT'], data=message))

    def send_enable_css_response(self, message):
        """
        Tries to enable a custom CSS file and sends the result to the extension.

        :param message string: the name of the CSS file to enable/disable
        """
        action = ACTIONS['CSS_ENABLE']
        target = self.check_target(message)
        if target is not False:
            if self.check_chrome_path(action):
                (success, message) = enable_custom_css(self.chrome_path, target)
                self.messenger.send_message(Message(
                    action,
                    data=target,
                    success=success,
                    message=message
                ))

    def send_disable_css_response(self, message):
        """
        Tries to disable a custom CSS file and sends the result to the extension.

        :param message string: the name of the CSS file to enable/disable
        """
        action = ACTIONS['CSS_DISABLE']
        target = self.check_target(message)
        if target is not False:
            if self.check_chrome_path(action):
                (success, message) = disable_custom_css(self.chrome_path, target)
                self.messenger.send_message(Message(
                    action,
                    data=target,
                    success=success,
                    message=message
                ))

    def send_font_size_response(self, message):
        """
        Tries to set a custom font size in a CSS file.

        :param message string: the name of the CSS file to change the font size in
        """
        action = ACTIONS['CSS_FONT_SIZE']
        target = self.check_target(message)
        if target is not False:
            if self.check_chrome_path(action):
                if 'size' in message:
                    new_size = message['size']
                    (success, message) = set_font_size(self.chrome_path, target, new_size)
                    self.messenger.send_message(Message(
                        action,
                        data=new_size,
                        success=success,
                        message=message
                    ))

    def handle_message(self, message):
        """
        Handles the incoming messages and does the appropriate action.

        :param message object: the decoded message
        """
        try:
            action = message['action']
            if action == ACTIONS['VERSION']:
                self.send_version()
            elif action == ACTIONS['COLORS']:
                self.send_colorscheme()
            elif action == ACTIONS['CSS_ENABLE']:
                self.send_enable_css_response(message)
            elif action == ACTIONS['CSS_DISABLE']:
                self.send_disable_css_response(message)
            elif action == ACTIONS['CSS_FONT_SIZE']:
                self.send_font_size_response(message)
            else:
                logging.debug('%s: no such action' % action)
                self.send_invalid_action()
        except KeyError:
            logging.error('action was not defined')
            self.send_invalid_action()

    def socket_thread_worker(self):
        """The socket server thread worker."""
        while True:
            message = self.socket_server.get_message()
            if message == 'update':
                logging.debug('Update triggered from external script')
                self.send_colorscheme()

    def start_socket_server(self):
        """Starts the socket server and creates the socket thread."""
        success = self.socket_server.start()
        if success == True:
            if self.python_version == 3:
                # We use daemon=True so that the thread will exit when the daemon exits.
                # https://docs.python.org/2/library/threading.html#threading.Thread.daemon
                self.socket_thread = Thread(target=self.socket_thread_worker, daemon=True)
            else:
                self.socket_thread = Thread(target=self.socket_thread_worker)

            self.socket_thread.start()

    def start(self):
        """Starts the daemon and listens for incoming messages."""
        self.is_running = True
        self.start_socket_server()
        try:
            while True:
                message = self.messenger.get_message()
                logging.debug('Received message from extension: %s' % message)
                self.handle_message(message)
        except KeyboardInterrupt:
            return

    def close(self):
        """Application cleanup."""
        self.socket_server.close()
        self.is_running = False
        logging.debug('Cleanup')













from __future__ import print_function

import os
import re
import glob
import shutil
import logging
import fileinput

from .config import CSS_PATH

def get_firefox_chrome_path():
    """
    Retrieves the path to the 'chrome' folder in the default Firefox profile

    :return: the absolute path to the chrome folder
    :rType: str
    """
    profile_path = glob.glob('%s/*.default-release' % os.path.expanduser('~/.mozilla/firefox'))

    if len(profile_path) < 1:
        logging.error('Could not find default Firefox profile')
        return False

    chrome_path = os.path.join(profile_path[0], 'chrome')
    if not os.path.exists(chrome_path):
        logging.debug('Creating non-existant chrome directory')
        os.makedirs(chrome_path)

    logging.debug('Found chrome directory at path: %s' % chrome_path)
    return chrome_path

def enable_custom_css(chrome_path, name):
    """
    Applies a CSS file but putting it in the 'chrome' directory.

    :param chrome_path str: the path to the chrome directory
    :param name str: the name of the css file to apply
    :return: (success, message)
    :rType: tuple
    """
    filename = add_css_file_extension(name)
    logging.debug('Enabling custom CSS file: %s' % filename)
    try:
        shutil.copy(os.path.join(CSS_PATH, filename), os.path.join(chrome_path, filename))
        logging.debug('%s was enabled' % filename)
        return (True, 'Custom CSS: %s has been enabled' % filename)
    except Exception as e:
        logging.error('%s could not be enabled: %s' % (filename, str(e)))
        return (False, 'Could not copy custom CSS to folder: %s' % str(e))

def disable_custom_css(chrome_path, name):
    """
    Disabled a CSS file but removing it from the 'chrome' directory.

    :param chrome_path str: the path to the chrome directory
    :param name str: the name of the css file to disable
    :return: (success, message)
    :rType: tuple
    """
    filename = add_css_file_extension(name)
    logging.debug('Disabling custom CSS file: %s' % filename)
    try:
        os.remove(os.path.join(chrome_path, filename))
        logging.debug('%s was disabled' % filename)
        return (True, 'Custom CSS: %s has been disabled' % filename)
    except Exception as e:
        logging.error('%s could not be disabled: %s' % (filename, str(e)))
        return (False, 'Could not remove custom CSS: %s' % str(e))

def set_font_size(chrome_path, name, size):
    """
    Sets the default font size in a CSS file.

    :param chrome_path str: the path to the chrome directory
    :param name str: the name of the css file to disable
    :param size int: the new font size
    :return: (success, message)
    :rType: tuple
    """
    filename = add_css_file_extension(name)
    logging.debug('Setting font size to %s in custom CSS file: %s' % (size, filename))
    try:
        for line in fileinput.input(os.path.join(chrome_path, filename), inplace=True):
            if '--pywalfox-font-size:' in line:
                print('  --pywalfox-font-size: %spx;' % size)
            else:
                print(line, end='')
        return (True, 'Font size was set to: %s' % size)
    except Exception as e:
        error_msg = 'Could not set font size: %s' % str(e)
        logging.error(error_msg)
        return (False, error_msg)

def add_css_file_extension(name):
    """
    Appends the CSS file extension to a string.

    :return: name with '.css' append at the end append at the end
    :rType: string
    """
    return '%s.css' % name
